version 8.0
log using formation.log, replace
#delimit ;

*     ***************************************************************** *;
*     ***************************************************************** *;
*     File-Name:  formation.do                                          *;
*     Date:       November 12, 2004                                     *;
*     Author:     SNG, using Martin and Stevenson's code from AJPS 2001 *;
*     Purpose:    Replicate results from AJPS 2001 article and do       *;
*           more rigorous tests of whether or not the IIA assumption    *;
*           holds.  Martin and Stevenson only drop 10 percent of their  *;
*           sample but this is may be too small given the proportion    *;
*           of zeros in their sample.  I took their code and put it     *;
*           together, with comments, to run these tests.                *;
*           Use MS data from 2001 article, with some new variables      *;
*           added (see "data_setup.do").                                *;
*     Data Used:  formation_new.dta                                     *;
*     Output File:    formation.log                                     *;
*     Machine:    laptop (compaq)                                       *;
*     ****************************************************************  *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*       Here are some of the original comments from the MS code         *;
*       (all their original comments will be in quotation marks)        *;
*     ****************************************************************  *;
* "This program performs a Hausman test for the violation of the IIA    *;
* assumption.  The format is iiatest varnames for the clogit model.     *;
* per is the percentage of the sample to keep, gid is the group         *;
* identifier for the clogit, reps is the number of times to do the      *;
* test, over which the average is taken."                               *;
*     ****************************************************************  *;

set matsize 150;
set mem 100m;
set more off;

use C:\datasets\formation_new.dta;

*     ****************************************************************  *;
*      "Note that if the stat is distributed chi-square then the        *;
*      average stat will asymptotically be distributed normally by CLT" *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*       The first part of their code creates a program, 'iiatest' that  *;
*       is supposed to run a Hausman test on each conditional logit     *;
*       model (11 in all). In footnote 8, page 39 of their article,     *;
*       they say that this is a standard statistical test proposed by   *;
*       Housman and McFadden 1984.  They want to see if the IIA         *;
*       assumption is a problem for their use of clogit.  They drop a   *;
*       random set of alternatives from each formation opportunity      *;
*       and then apply the Hausman test.  In the article, they have     *;
*       an average of 150 zeros and 1 one for each formation            *;
*       opportunity.  They drop only 10 percent of the 150 choices      *;
*       each time they apply the Hausman test.                          *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*       'iiatest' is a program to run the models and check for IIA      *;
*     ****************************************************************  *;

capture program drop iiatest;
program define iiatest;

*  the syntax line tells the computer to look for the following format  *;
*  Stata will examine what is typed and try to match it to this.        *;

    syntax varlist [if], per(integer) gid(varname) reps(integer);
    preserve;

*  create matrices for chi-square, degrees of freedom for the statistic *;
*  and the p-value for the chi-square.                                  *;
*  The matrix CHLIST, for example, is defined as a `reps' x 1 matrix    *;
*  containing elements 0.                                               *;

    matrix CHLIST=J(`reps',1,0); 
    matrix DFLIST=J(`reps',1,0);
    matrix PRLIST=J(`reps',1,0);

*  `tokenize' divides string into tokens, storing the result in the     *;
*   positional local macro `varlist'.  `local' assigns strings to local *;
*   macro names.                                                        *;

    tokenize `varlist';
    local realg "`1'";
    macro shift 1;
    local ivs "`*'";

*   the next command will run conditional logit on any model in the     *;
*   appropriate format                                                  *;

    clogit `realg' `ivs' `if', group(`gid');
    qui hausman, save;
    local i `reps';
    while `i' > 0 {;
        tempfile master;
        qui save `master';
        display "rep number `i'";
        qui sample `per' if `realg'==0, by(`gid');
        qui clogit `realg' `ivs' `if', group(`gid');
        qui hausman, alleqs less ;       
        qui mat DFLIST[`i',1]=`r(df)';
        if `r(chi2)' > 0 { ;        
            qui mat PRLIST[`i',1]=chiprob(`r(df)',`r(chi2)');
            qui mat CHLIST[`i',1]=`r(chi2)';
        };
        else if `r(chi2)' <= 0 {;
            qui mat PRLIST[`i',1]=0.99;
            qui mat CHLIST[`i',1]=0.0;
        };
        local i=`i'-1;
        use `master', clear;

    };

mat list CHLIST;
mat CHMAT=diag(CHLIST);
scalar avstat=trace(CHMAT)/`reps';
scalar list avstat;

mat list DFLIST;
mat DFMAT=diag(DFLIST);
scalar avdf=trace(DFMAT)/`reps';
scalar list avdf;

mat list PRLIST;
mat PRMAT=diag(PRLIST);
scalar avpr=trace(PRMAT)/`reps';
scalar list avpr;

end;

*     ****************************************************************  *;
*       What follows is the second of the two do files provided by      *;
*       Martin and Stevenson.                                           *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*       "NOTE:  For replication, you must first open formation03.dta    *; 
*       (you should set mem 10m first), then run iiatest.do before      *;
*       running this program."                                          *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*       model 1 (this is Martin and Stevenson's Model 10)               *;
*     ****************************************************************  *;
*       note that "warmiss" is 1 if the data from Warwick is missing    *;
*     ****************************************************************  *;

iiatest realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq mginvest anmax2 pro vsp vspsing msp 
mspsing if warmiss~=1, per(50) gid(newid) reps(50);

*     ****************************************************************  *;
*       model 2 (this is their model but with constituent term,         *;
*       invest, added)                                                  *;
*     ****************************************************************  *;

iiatest realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro vsp singpar 
vspsing msp mspsing if warmiss~=1, per(50) gid(newid) reps(50);

*     ****************************************************************  *;
*       model 3 (here we drop all formation opportunities that do not   *;
*       occur after an election)                                        *;
*     ****************************************************************  *;

iiatest realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro vsp singpar 
vspsing msp mspsing if warmiss~=1 & election==1, per(50) gid(newid) reps(50);

*     ****************************************************************  *;
*       model 4 (only governments forming after elections, the          *;
*       constituent term added, revised pre-electoral coalition (pec)   *;
*       variable included based on dataset from Golder)                 *;
*     ****************************************************************  *;

iiatest realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pec vsp singpar 
vspsing msp mspsing if warmiss~=1 & election==1, per(50) gid(newid) reps(50);


*     ****************************************************************  *;
*       Using Model 2, calculate some conditional coefficients and      *;
*       conditional standard errors (this is on the latent dependent    *;
*       variable, though)                                               *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*         effect of very strong party + (VSP*singleparty)               *;
*     ****************************************************************  *;

clogit realg vsp singpar vspsing minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro msp mspsing if warmiss~=1, group(case);


matrix b=e(b);
matrix V=e(V);
scalar b1=b[1,1];
scalar b2=b[1,2];
scalar b3=b[1,3];
scalar varb1=V[1,1];
scalar varb3=V[3,3];
scalar covb1b3=V[1,3];
gen beta1=b1;
gen beta3=b3;
gen conb1=b1+b3*1;
gen conb0=b1+b3*0;
gen conse1=sqrt(varb1+varb3*1^2+2*covb1b3*1);
gen conse0=sqrt(varb1+varb3*0^2+2*covb1b3*0);
gen a1=1.96*conse1;
gen top1=conb1+a1;
gen bottom1=conb1-a1;
gen a0=1.96*conse0;
gen top0=conb0+a0;
gen bottom0=conb0-a0;

sum beta1 beta3 conb1 top1 bottom1 conb0 top0 bottom0;

*     ****************************************************************  *;
*       effect of merely strong party + (MSP*singleparty)               *;
*     ****************************************************************  *;


clogit realg msp singpar mspsing minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro vsp vspsing if warmiss~=1, group(case);


matrix b=e(b);
matrix V=e(V);
scalar b1m=b[1,1];
scalar b2m=b[1,2];
scalar b3m=b[1,3];
scalar var1m=V[1,1];
scalar var3m=V[3,3];
scalar covb1b3m=V[1,3];
gen beta1m=b1m;
gen beta3m=b3m;
gen conb1m=b1m+b3m*1;
gen conb0m=b1m+b3m*0;
gen conse1m=sqrt(var1m+var3m*1^2+2*covb1b3m*1);
gen conse0m=sqrt(var1m+var3m*0^2+2*covb1b3m*0);
gen a1m=1.96*conse1m;
gen top1m=conb1m+a1m;
gen bottom1m=conb1m-a1m;
gen a0m=1.96*conse0m;
gen top0m=conb0m+a0m;
gen bottom0m=conb0m-a0m;

sum beta1m beta3m conb1m top1m bottom1m conb0m top0m bottom0m;


*     ****************************************************************  *;
*       effect of merely strong & single party                          *;
*     ****************************************************************  *;

clogit realg singpar msp vsp mspsing vspsing minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro if warmiss~=1, group(case);


matrix b=e(b);
matrix V=e(V);
scalar b1sm=b[1,1];
scalar b4sm=b[1,4];
scalar var1sm=V[1,1];
scalar var4sm=V[4,4];
scalar covb1b4sm=V[1,4];
gen beta1sm=b1sm;
gen beta4sm=b4sm;
gen conb1sm=b1sm+b4sm*1;
gen conb0sm=b1sm+b4sm*0;
gen conse1sm=sqrt(var1sm+var4sm*1^2+2*covb1b4sm*1);
gen conse0sm=sqrt(var1sm+var4sm*0^2+2*covb1b4sm*0);
gen a1sm=1.96*conse1sm;
gen top1sm=conb1sm+a1sm;
gen bottom1sm=conb1sm-a1sm;
gen a0sm=1.96*conse0sm;
gen top0sm=conb0sm+a0sm;
gen bottom0sm=conb0sm-a0sm;

sum beta1sm beta4sm conb1sm top1sm bottom1sm conb0sm top0sm bottom0sm;


*     ****************************************************************  *;
*       effect of very strong & single party                            *;
*     ****************************************************************  *;

clogit realg singpar msp vsp mspsing vspsing minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro if warmiss~=1, group(case);

matrix b=e(b);
matrix V=e(V);
scalar b1sv=b[1,1];
scalar b5sv=b[1,5];
scalar var1sv=V[1,1];
scalar var5sv=V[5,5];
scalar covb1b5sv=V[1,5];
gen beta1sv=b1sv;
gen beta5sv=b5sv;
gen conb1sv=b1sv+b5sv*1;
gen conb0sv=b1sv+b5sv*0;
gen conse1sv=sqrt(var1sv+var5sv*1^2+2*covb1b5sv*1);
gen conse0sv=sqrt(var1sv+var5sv*0^2+2*covb1b5sv*0);
gen a1sv=1.96*conse1sv;
gen top1sv=conb1sv+a1sv;
gen bottom1sv=conb1sv-a1sv;
gen a0sv=1.96*conse0sv;
gen top0sv=conb0sv+a0sv;
gen bottom0sv=conb0sv-a0sv;

sum beta1sv beta5sv conb1sv top1sv bottom1sv conb0sv top0sv bottom0sv;



*     ****************************************************************  *;
*       Getting a measure of overall predictive accuracy: how often     *;
*       is the alternative that is the most likely to form the          *;
*       government actually the one that gets chosen?  We'll try this   *;
*       on each of the models.                                          *;
*     ****************************************************************  *;

*     ****************************************************************  *;
*                       model 1                                         *;
*     ****************************************************************  *;

clogit realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq mginvest anmax2 pro vsp vspsing msp 
mspsing if warmiss~=1, group(case);

predict phat if e(sample);
sum phat if realg==1 & warmiss~=1;
sum phat if realg==0 & warmiss~=1;
sort case phat;
gen success=0;
by case: generate caseID=_n;
by case: egen maxcase = max(caseID);
replace success = 1 if maxcase==caseID & realg==1 & warmiss~=1;
sum success if success==1;

*     ****************************************************************  *;
*       Take the last number generated and divide by 170 (the number    *;
*       of formation opportunities).  This gives you an estimate of     *;
*       the predictive success of the model.                            *;
*     ****************************************************************  *;

drop phat success caseID maxcase;




*     ****************************************************************  *;
*                       model 2                                         *;
*     ****************************************************************  *;

clogit realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro vsp singpar 
vspsing msp mspsing if warmiss~=1, group(case);

predict phat if e(sample);
sum phat if realg==1 & warmiss~=1;
sum phat if realg==0 & warmiss~=1;
sort case phat;
gen success=0;
by case: generate caseID=_n;
by case: egen maxcase = max(caseID);
replace success = 1 if maxcase==caseID & realg==1 & warmiss~=1;
sum success if success==1;
sum success if success==1 & election==1;
sum success if success==1 & election==0;

*     ****************************************************************  *;
*       Take the last number generated and divide by 170 (the number    *;
*       of formation opportunities).  This gives you an estimate of     *;
*       the predictive success of the model.                            *;
*     ****************************************************************  *;

drop phat success caseID maxcase;



*     ****************************************************************  *;
*                       model 3                                         *;
*     ****************************************************************  *;


clogit realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pro vsp singpar 
vspsing msp mspsing if warmiss~=1 & election==1, group(case);


predict phat if e(sample);
sum phat if realg==1 & warmiss~=1 & election==1;
sum phat if realg==0 & warmiss~=1 & election==1;
sort case phat;
gen success=0;
by case: generate caseID=_n;
by case: egen maxcase = max(caseID);
replace success = 1 if maxcase==caseID & realg==1 & warmiss~=1 & election==1;
sum success if success==1;

*     ****************************************************************  *;
*       Take the last number generated and divide by 96 (the number     *;
*       of formation opportunities).  This gives you an estimate of     *;
*       the predictive success of the model.                            *;
*     ****************************************************************  *;

drop phat success caseID maxcase;






*     ****************************************************************  *;
*                       model 4                                         *;
*     ****************************************************************  *;


clogit realg minor minwin numpar dompar median gdiv1 
mgodiv1 prevpm sq invest mginvest anmax2 pec vsp singpar 
vspsing msp mspsing if warmiss~=1 & election==1, group(case);


predict phat if e(sample);
sum phat if realg==1 & warmiss~=1 & election==1;
sum phat if realg==0 & warmiss~=1 & election==1;
sort case phat;
gen success=0;
by case: generate caseID=_n;
by case: egen maxcase = max(caseID);
replace success = 1 if maxcase==caseID & realg==1 & warmiss~=1 & election==1;
sum success if success==1;

*     ****************************************************************  *;
*       Take the last number generated and divide by 96 (the number     *;
*       of formation opportunities).  This gives you an estimate of     *;
*       the predictive success of the model.                            *;
*     ****************************************************************  *;

drop phat success caseID maxcase;

log close;

exit;
